## In this file, we load the original datasets from multiple sample sites and concatenate them into 
## a single data matrix using the function matcat. 
## In line 40, we define the number of samples from each site+subject combo for random sampling on lines 58 & 59

site_set <- c("female_feces","male_feces","male_tongue","female_tongue")

matcat <- function(A,B){
  if (typeof(B) != 'double' || typeof(A) != 'double'){stop('input must be two matrices')}
  
  p1 <- dim(A)[2]
  p2 <- dim(B)[2]
  
  m1 <- dim(A)[1]
  m2 <- dim(B)[1]
  
  C <- A
  #First, we'll bind the overlaps
  dum <- matrix(0,nrow=m1,ncol=p2)
  ix1 <- rownames(A)
  ix2 <- rownames(B)
  overlap <- intersect(ix1,ix2)
  rownames(dum) <- ix1
  colnames(dum) <- colnames(B)
  ind1 <- match(ix2[ix2 %in% overlap],ix1) 
  ind2 <- match(ix1[ix1 %in% overlap],ix2)
  dum[ind1,] = B[ind2,]
  C <- cbind(C,dum)
  
  #Then, we'll bind the non-overlaps
  D <- B[!(ix2 %in% ix1),]
  dum <- matrix(0,nrow=dim(D)[1],ncol=p1)
  colnames(dum)<- colnames(A)
  rownames(dum) <- rownames(D)
  D <- cbind(dum,D)
  C <- rbind(C,D)
  return(C)
}


p=10 #number of samples per site
Dat = NULL #Our Data matrix
X=NULL # Our independent variable
for (ss in 1:length(site_set)){
  
  
  samplesite <- site_set[ss]
  if (ss<=2){
    X <- c(X,rep("feces",p))
  } else {X <- c(X,rep("tongue",p))}
  
  ## Now we load site data and configure
  setwd(paste("C:/Users/Big Al/Documents/",samplesite,sep=''))
  load(paste(samplesite,'_phylo_analysis.r',sep=''))
  #   C <- otutable_site %>% t%>% clo %>% clr %>% t %>% cov
  #   image(C,main=samplesite)
  
  if (is.null(p)==F){ #possibly sub-sample otutable
    sbs <- sample(dim(otutable_site)[2],p) #subset of full dataset
    otutable_site <- otutable_site[,sbs]
  }
  
  if (ss==1){
    Dat <- as.matrix(otutable_site)
  } else {
    Dat <- matcat(Dat,as.matrix(otutable_site))
  }
}
#Finally, we remove zero-rows
Dat <- Dat[which(rowSums(Dat)>0),]
X <- as.factor(X)



### Now we'll trim data to only the most abundant taxa
numTaxa <- rep(0,dim(Dat)[2])
PercZeros <- numTaxa
for (ii in 1:length(numTaxa)){
  numTaxa[ii] <- sum(rowSums(Dat==0)<ii)
  PercZeros[ii] <- sum(Dat[which(rowSums(Dat==0)<ii),]==0)/(length(which(rowSums(Dat==0)<ii))*dim(Dat)[2])
}

# We're going to use only those taxa which were present in >= 30 samples
ind <- which(rowSums(Dat==0)<30)
Dat <- Dat[ind,]
